# Url Shortener Library.

from werkzeug.routing import BaseConverter


class RegexConverter(BaseConverter):
    def __init__(self, url_map, *items):
        super(RegexConverter, self).__init__(url_map)
        self.regex = items[0]


class UrlShortener(object):
    """This object is used to hold the settings used for short url. Instances
    of :class:`UrlShortner` are *not* bound to specific apps, so you can
    create one in the main body of your code and then bind it to your
    app in a factory function.
    """

    def __init__(self, app=None, add_context_processor=True):
        if app is not None:
            self.init_app(app, add_context_processor)

    def setup_app(self, app, add_context_processor=True):  # pragma: no cover
        """
        This method has been deprecated. Please use
        :meth:`LoginManager.init_app` instead.
        """
        self.init_app(app, add_context_processor)

    def init_app(self, app, add_context_processor=True):
        """
        Configures an application.
        :param app: The :class:`flask.Flask` object to configure.
        :type app: :class:`flask.Flask`
        :param add_context_processor: Whether to add a context processor to
            the app that adds a `current_user` variable to the template.
            Defaults to ``True``.
        :type add_context_processor: bool
        """

        app.url_map.converters['regex'] = RegexConverter

        app.login_manager = self
        from lil_url.url_shortner import server_url
        from lil_url import DEFAULT_PREFIX
        app.add_url_rule('/{}/<regex("[a-zA-Z0-9_-]*"):slug>/'.format(DEFAULT_PREFIX), 'serve_short_url', server_url)



from lil_url.url_shortner import shorten_url
from lil_url.libs.url_shortner import UrlShortener

DEFAULT_PREFIX = 'little_url'


def init_app(app):
    shortener = UrlShortener()
    shortener.init_app(app)



import random
import string

from flask import Blueprint, redirect, request

from lil_url.helpers.redis_helper import get_redis


def shorten_url(url, url_slug=None, expiry=None, force_new=False):
    """Shortens the real URL to short URL.

    Parameters
    ----------
    `url`: <String>
        Long URL which is to be shorten.

    url_slug: <String>
        If any chosen slug to be served.

    expiry: <Int>
        expiry time in seconds.

    Returns
    -------
    <Dict>:
        Returns respnse dict.
    """

    response = {"success": False, "message": "", "code": "", "slug": "", "short_url": ''}
    from flask import current_app
    from lil_url import DEFAULT_PREFIX

    host, port = None, None
    if current_app.config.get("REDIS_HOST", None):
        host, port = current_app.config.get("REDIS_HOST"), current_app.config.get("REDIS_PORT", None)

    redis = get_redis(host, port)

    if url_slug:
        if redis.exists(url_slug):
            response["message"] = "The slug already exists. Try with a new one."
    else:
        if redis.exists(url):
            url_slug = redis.get(url)
        else:
            url_slug = _generate_url_slug()

    redis.set(url_slug, url)
    redis.set(url, url_slug)

    if expiry:
        redis.expire(url_slug, expiry)

    response["success"] = True
    response["slug"] = url_slug
    response["short_url"] = '{}/{}'.format(current_app.config.get('BASE_URL'), url_slug)

    return response


def server_url(slug):
    """Serves the short URL to new one.

    Parameters
    ----------
    `slug`: <String>
        Slug for which long URL to be served.

    Returns
    -------
        Redirects to the real/long URL.
    """

    host, port = None, None
    from flask import current_app
    if current_app.config.get("REDIS_HOST", None):
        host, port = current_app.config.get("REDIS_HOST", None), current_app.config.get('REDIS_PORT')

    redis = get_redis(host, port)

    if redis.exists(slug):
        return redirect(redis.get(slug))
    else:
        return redirect(redis.get(slug))


def _generate_url_slug(size=10, chars=string.ascii_lowercase + string.digits):
    """
    This is for a Django project and it assumes your instance
    has a model with a slug field and a title character (char) field.

    Parameters
    ----------
    size: <Int>
        Size of the slug.

    chars: <string.class>
        Character class to be included in the slug.
    """

    slug = ''.join(random.choice(chars) for _ in range(size))
    from flask import current_app

    host, port = None, None
    if current_app.config.get("REDIS_HOST", None):
        host, port = current_app.config.get("REDIS_HOST", None), current_app.config.get("REDIS_PORT", None)

    redis = get_redis(host, port)

    if redis.exists(slug):
        try:
            return _generate_url_slug()
        except RecursionError:
            return
    else:
        return slug



from __future__ import print_function
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import io
import os
import sys

here = os.path.abspath(os.path.dirname(__file__))
LONG_DESCRIPTION = 'Url Shortener for flask.'
# with open('README.rst', 'r') as f:
#     LONG_DESCRIPTION = f.read()
CLASSIFIERS = filter(None, map(str.strip,
"""
Development Status :: 2 - Pre-Alpha
Intended Audience :: Developers
License :: OSI Approved :: MIT License
License :: OSI Approved :: Academic Free License (AFL)
Programming Language :: Python
Programming Language :: Python :: 2
Programming Language :: Python :: 2.5
Programming Language :: Python :: 2.6
Programming Language :: Python :: 2.7
Programming Language :: Python :: 3.5
Programming Language :: Python :: 3.6
Programming Language :: Python :: Implementation :: CPython
Programming Language :: Python :: Implementation :: PyPy
Topic :: Software Development :: Libraries :: Python Modules
""".splitlines()))

EXCLUDES = ['contrib', 'docs', 'tests', 'htmlcov', 'docs', '.cache']


def read(*filenames, **kwargs):
    encoding = kwargs.get('encoding', 'utf-8')
    sep = kwargs.get('sep', '\n')
    buf = []
    for filename in filenames:
        with io.open(filename, encoding=encoding) as f:
            buf.append(f.read())
    return sep.join(buf)


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errcode = pytest.main(self.test_args)
        sys.exit(errcode)

setup(
    name="LilUrl",
    include_package_data=True,
    version='0.0.10',
    description='',
    long_description=LONG_DESCRIPTION,
    classifiers=CLASSIFIERS,
    author="Om Prakash",
    author_email="oppradhan2011@gmail.com",
    url="https://github.com/omprakash1989/LilUrl",
    license="MIT License",
    platforms=['any'],
    extras_require={
        'dev': ['check-manifest'],
        'test': ['coverage'],
    },
    # What does your project relate to?
    keywords='Url Shortner.',
    tests_require=['pytest'],
    setup_requires=['pytest-runner'],

    packages=find_packages(exclude=EXCLUDES),

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        'url_shortener': ['url_shortener.dat'],
    },

    install_requires=['coverage', 'redis'],

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[('url_shortener_data', ['data/data_file'])],

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
)
